-- Created by: Danny Bloemendaal, danny@bloemeland.nl
-- Version 0.9.1

-- make sure we can find our modules
local SCRIPTFOLDER = 'scripts'
local scriptPath = debug.getinfo(1).source:match("@?(.*/)")
package.path    = package.path .. ';' .. scriptPath .. '?.lua'
package.path    = package.path .. ';' .. scriptPath .. SCRIPTFOLDER .. '/?.lua'


local MAIN_METHOD = 'execute'

function callEventHandler(eventHandler, value)
	if (eventHandler[MAIN_METHOD] ~= nil) then
		local ok2, res, continue = pcall(eventHandler[MAIN_METHOD], value)
		if (ok2) then
			return res, continue
		else
			print('An error occured when calling event handler ' .. eventHandler)
			print(res) -- error info
		end
	else
		print('No' .. MAIN_METHOD .. 'function found in event handler ' .. eventHandler)
	end
	return {}, true
end

function getIndex(device, otherdevices_idx)
	if (otherdevices_idx == nil) then
		-- old system
		return 0
	end

	-- first get the name, it could be 'My temperature_humidity', we only need the part before the _
	local name = device
	local pos, len = string.find(device, '_')
	if (pos ~= nil and pos > 1) then -- cannot start with _ (we use that for our _always script)
		name = string.sub(device, 1, pos-1)
	end
	-- now find the index in the table otherdevices_idx
	local idx = otherdevices_idx[name]

	return idx
end

function handleEvents(events, deviceValue, commandArray, commandIndex, continue)
	local commands

	if (type(events) ~= 'table') then
		return continue
	end

	if (continue) then
		for eventIdx, eventHandler in pairs(events) do
			print('=====================================================')
			print('>>> Handler: ' .. eventHandler.name )
			print('.....................................................')

			commands, continue = callEventHandler(eventHandler, deviceValue)

			print('.....................................................')
			print('<<< Done ')
			print('=====================================================')

			-- commandIndex = commandIndex + 1
			for k,v in pairs(commands) do
				--commandArray[k] = v
				commandIndex = commandIndex + 1
				commandArray[commandIndex] = {}
				commandArray[commandIndex][k]=v
			end

			-- normalize return value
			if (continue==nil) then continue = true end -- continue

			if (continue == false) then break end -- no need to go on here
		end
	end

	return continue, commandArray, commandIndex
end

function getEventBindings(mode)
	local bindings = {}
	local ok, modules, moduleName, i, event, j
	ok, modules = pcall( scandir, scriptPath .. '/' .. SCRIPTFOLDER)
	if (not ok) then
		print(modules)
		return nil
	end

	for i, moduleName  in pairs(modules) do
		local module
		ok, module = pcall(require, moduleName)
		print(ok)
		if (ok) then
			if (type(module) == 'table') then
				if ( module.on ~= nil and module[MAIN_METHOD]~= nil ) then
					module.name = moduleName
					for j, event in pairs(module.on) do
						if (mode == 'timer') then
							if (event == 'timer') then
								table.insert(bindings, module)
							end
						else
							if (event ~= 'timer') then
								if ( bindings[event] == nil) then
									bindings[event] = {}
								end
								table.insert(bindings[event], module)
							end
						end
					end
				else
					print('Script ' .. moduleName .. '.lua has no "on" and/or "' .. MAIN_METHOD .. '" section. Skipping')
				end
			else
				print('Script ' .. moduleName .. '.lua is not a valid module. Skipping')
			end
		else
			print(module)
		end
	end

	return bindings
end

function getTimerHandlers()
	return getEventBindings('timer')
end

function scandir(directory)
	local pos, len
	local i, t, popen = 0, {}, io.popen
	local sep = string.sub(package.config,1,1)
	local cmd

	if (sep=='/') then
		cmd = 'ls -a "'..directory..'"'
	else
		-- assume windows for now
		cmd = 'dir "'..directory..'" /b /ad'
	end

	local pfile = popen(cmd)
	for filename in pfile:lines() do

		pos,len = string.find(filename, '.lua')

		if (pos and pos > 0 ) then
			i = i + 1
			t[i] = string.sub(filename, 1, pos-1)
		end

	end
	pfile:close()
	return t
end


function dumpCommandArray(commandArray)
	for k,v in pairs(commandArray) do
		if (type(v)=='table') then
			for kk,vv in pairs(v) do
				print('[' .. k .. '] = ' .. kk .. ': ' .. vv)
			end
		else
			print(k .. ': ' .. v)
		end
	end
end

return {
	callEventHandler = callEventHandler,
	getIndex = getIndex,
	handleEvents = handleEvents,
	getEventBindings = getEventBindings,
	getTimerHandlers = getTimerHandlers,
	dumpCommandArray = dumpCommandArray
}
